%-------------------------------------------------------%
% Decomposition into texture and geometry components    %
% Program for data degraded by a blur and Poisson noise %
%                                                       %
% For theoretical aspects please refer to :             %
% http://www-syscom.univ-mlv.fr/~pustelni/theseE.html   %
%-------------------------------------------------------%

clc
close all
clear all
addpath DTT/filters
addpath DTT/dual-tree
addpath Images
addpath include_textGeom
addpath include_hybrid
addpath include

% Degradation parameters
alpha= 0.5;                 % Scaling parameter for Poisson noise
L=3;                        % Kernel size blur

% DTT parameters
M='sym3';                   % Filter   
rm=3;                       % Decomposition level


% Parallel ProXimal Algorithm parameters
niter=150;                   % Iterations number   


%Load image
load sunflower.mat
nomImage = 'sunflower';      %Image name to data registration


%Data degradation
[z,T,h] = bruiteConv(x,L,alpha);


% Convex constraints bounds
minx=min(x(:));
maxx=max(x(:));

[wpx,wmx,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(x,M,rm);
[mup,mum] = estimMoyenne_DTT(wpx,wmx,rm);
Ml=length(wpx{1});
Mc=length(wpx{1}{1});
[Nx,Mx] = size(x);


% Decomposition into texture and geometry components
cte_app=1000;
cte_l1 = 0.03;             % texture regularization parameter
cte_tv = 0.025;             % geometric regularization parameter
coef_reg = [cte_l1,cte_tv];
xrec1 = PPXA_Poisson_TextGeom_TV_L1(x,Nx,Mx,z,alpha,L,T,mup,mum,2,M,rm,Ml,Mc,coef_reg,cte_app,minx,maxx,niter,nomImage) ;  



% Hybrid PPXA ( http://arxiv.org/pdf/0911.1536) to compare the restored
% image obtained with TextGeom decomposition method
cte_app=0.01;
cte_l1 = 0.01;              % l1 regularization parameter
cte_tv = 0.008;             % TV regularization parameter
coef_reg = [cte_l1,cte_tv];        
xrec2 = PPXA_Poisson_Hybrid_TV_L1(x,Nx,Mx,z,alpha,L,T,h,mup,mum,2,M,rm,Ml,Mc,coef_reg,cte_app,minx,maxx,niter,nomImage);


% Print results
figure
subplot(221)
imagesc(x)
title('Original');
axis image off
subplot(222)
imagesc(z)
title('Degraded');
axis image off
subplot(223)
imagesc(xrec2)
axis image off
title('Hybrid restoration');
subplot(224)
imagesc(xrec1)
axis image off
title('TextGeo restoration');
colormap(gray)
