function xrec = PPXA_Poisson_Hybrid_TV_L1(x,Nx,Mx,z,alpha,L,T,h,mup,mum,mu,M,rm,Ml,Mc,coef,coef_app,minx,maxx,niter,nomImage)


%Number of functional 
%--------------------
L_old=L;
if (L==1) L=1;
elseif (L==3) L=4;    
elseif (L==5) L=8;
elseif (L==7) L=8; 
elseif (L==9) L=16; 
elseif (L==11) L=16;   
elseif (L==13) L=16;  
elseif (L==15) L=16; 
end

flou = L*L;     %Data fidelity term
reg = 1;        %Frame regularization
cc = 1;         %Convex constraint
tv=4;           %Total variation [Combettes&Pesquet2008]

N1=flou +reg +cc +tv;
fprintf('\n Proximity operators number \t: %d\n',N1);
fprintf('----------------------------\n');
fprintf('Proximity operators related to data fidelity term \t: %d\n',flou);
fprintf('Proximity operators related to the frame regularization term \t: %d\n',reg);
fprintf('Projection onto the convex constraint \t: %d\n',cc);
fprintf('Proximity operators related to TV \t: %d\n',tv);


%alpha_i value
%-------------
coef_alpha=1/(L_old*L_old);


%Algorithm initialization
%------------------------
xrec=z;
[wzp,wzm,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(z,M,rm);
wzp_vect = cell2vect(wzp,rm,Ml,Mc);
wzm_vect = cell2vect(wzm,rm,Ml,Mc);

wynip = ones(1,1)*wzp_vect;
wynim = ones(1,1)*wzm_vect;

wpnip = ones(1,1)*zeros(size(wzp_vect));
wpnim = ones(1,1)*zeros(size(wzm_vect));

zrec = idualtreeM2D_freq_deblur(wzp,wzm,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
zni = ones(N1-1,1)*zrec(:)';

[FFstar_wzp,FFstar_wzm,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(zrec,M,rm);
FFstar_wzp_vect = cell2vect(FFstar_wzp,rm,Ml,Mc);
FFstar_wzm_vect = cell2vect(FFstar_wzm,rm,Ml,Mc);

wynip_ortho = ones(N1-1,1)*(wzp_vect - 1/mu * FFstar_wzp_vect);
wynim_ortho = ones(N1-1,1)*(wzm_vect - 1/mu * FFstar_wzm_vect);

qni = ones(N1-1,1)*zeros(size(wzp_vect));


%Weighted constants initialization
%---------------------------------
wi = 1/N1*ones(N1,1);
wi = ones(N1,1)/(L*L*4);
wi(end)=1/4;
wi(flou+1)=1/4;
wi(flou+2:flou+5)=1/(4*4);


if (sum(wi)>1.0001 || sum(wi)<0.9999 ) 
    fprintf('ERREUR: The wi''s is non equal to 1. The value is %3.7f\n\n',sum(wi));
end

% wxn initialization
%-------------------
wxnp_temp = wynip;
wxnm_temp = wynim;
wxnp = sum(wxnp_temp,1);
wxnm = sum(wxnm_temp,1);



% Criterion initialization
%-------------------------
snr = zeros(1,niter);
divKL=zeros(1,niter);
sumcoef=zeros(1,niter);
tv=zeros(1,niter);
dc=zeros(1,niter);
critere=zeros(1,niter);
time = zeros(1,niter+1);

fprintf('\n Start PPXA \n'); 
fprintf('------------- \n');

%Algorithm
%---------
lambda=1;
gamma= 100;
time(1)=0;
for n=1:niter
   
    wxnpcell = vect2cell(wxnp,rm,Ml,Mc,Nx);
    wxnmcell = vect2cell(wxnm,rm,Ml,Mc,Nx);
   
    sumcoef(n)=sumcoef_l1m(wxnpcell,1,1,mup,coef(1),coef_app) + sumcoef_l1m(wxnmcell,1,1,mum,coef(1),coef_app);
    divKL(n) = DKL(z,xrec,T,alpha) ;
    snr(n) = snrdeblur(x,xrec);
    tv(n) = Variation_totale(xrec) ;
    dc(n) = distance_convexe(xrec,minx,maxx);
    critere(n)= divKL(n) + sumcoef(n) +coef(2)*tv(n);
    if(mod(n,10)==0) 
        fprintf('%d : dc = %3.2f  |  critere = %3.2f |  snr = %3.2f \n',n,dc(n),critere(n), snr(n));
    end
    
    % Proximity operators computation
    %--------------------------------
    
    tic;  
    
    %relative to data fidelity term
    for i=1:flou
        ind = i;
        j = floor((ind-1)/L);
        k=mod((ind-1),L);
        
        j1 = find(mod(1:floor(Nx/L)*L,L)==mod(j+1,L));
        k1 = find(mod(1:floor(Nx/L)*L,L)==mod(k+1,L));
        [qni(ind,:)]=1/mu*prox_poisson_partiel_DTTl1_TV(zni(ind,:),z,h,alpha,wi(ind,1),gamma,coef_alpha,j,k,j1,k1,mu,L_old,L);
    end
      
    % relative to convex constraint projection
    ind = flou + 1;
    [qni(ind,:)]=1/mu*projectionDTT(zni(ind,:),minx,maxx);
 
    % relative to total variation  
   [qni(flou+2,:)] = 1/mu*prox_tv_partiel(zni(flou+2,:),Nx,0,0,wi(flou+2,:),gamma,coef(2),mu);
   [qni(flou+3,:)] = 1/mu*prox_tv_partiel(zni(flou+3,:),Nx,0,1,wi(flou+3,:),gamma,coef(2),mu);
   [qni(flou+4,:)] = 1/mu*prox_tv_partiel(zni(flou+4,:),Nx,1,0,wi(flou+4,:),gamma,coef(2),mu);
   [qni(flou+5,:)] = 1/mu*prox_tv_partiel(zni(flou+5,:),Nx,1,1,wi(flou+5,:),gamma,coef(2),mu);
 
    % relative to frame regularization
    ind = N1;  
    wynip_regcell = vect2cell(wynip(1,:),rm,Ml,Mc,Nx);
    wynim_regcell = vect2cell(wynim(1,:),rm,Ml,Mc,Nx);
     for j=1:rm,
         for ml=1:Ml,
             for mc=1:Mc,
                 if (j==rm && ml==1 && mc==1),
                    % Approximation frame coefficients
                    omegap = coef_app*gamma/wi(ind,1);
                    wynip_regcell{j}{ml}{mc}= mup + prox_soft(wynip_regcell{j}{ml}{mc}-mup,omegap);
                    omegam = coef_app*gamma/wi(ind,1);
                    wynim_regcell{j}{ml}{mc}= mum + prox_soft(wynim_regcell{j}{ml}{mc}-mum,omegam);
                 
                    
                  elseif ~isempty(wynim_regcell{j}{ml}{mc})
                    % Details frame coefficients                     
                    omegap = coef(1)*gamma/wi(ind,1);
                    wynip_regcell{j}{ml}{mc}= prox_soft(wynip_regcell{j}{ml}{mc},omegap);
                    omegam = coef(1)*gamma/wi(ind,1);
                    wynim_regcell{j}{ml}{mc}= prox_soft(wynim_regcell{j}{ml}{mc},omegam);

                 end
              end
          end
     end
    wpnip(1,:) = cell2vect(wynip_regcell,rm,Ml,Mc);
    wpnim(1,:) = cell2vect(wynim_regcell,rm,Ml,Mc); 
    
    
    % Weighted sum
    %-------------    
    pn_temp1 = (wi(1:N1-1)*ones(1,length(qni))).*qni;
    pn_temp1_img = reshape(sum(pn_temp1,1),Nx,Nx);
    [wpnp1_cell,wpnm1_cell,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(pn_temp1_img,M,rm);
    wpnp1 = cell2vect(wpnp1_cell,rm,Ml,Mc);
    wpnm1 = cell2vect(wpnm1_cell,rm,Ml,Mc);  
        
    wpnp_temp2 = (wi(1:N1-1)*ones(1,length(wynip_ortho))).*wynip_ortho;
    wpnp2 = sum(wpnp_temp2,1);
    wpnm_temp2 = (wi(1:N1-1)*ones(1,length(wynim_ortho))).*wynim_ortho;
    wpnm2 = sum(wpnm_temp2,1);   

    wpnp_temp3 = (wi(N1)*ones(1,length(wpnip(1,:)))).*wpnip(1,:);
    wpnp3 = sum(wpnp_temp3,1);
    wpnm_temp3 = (wi(N1)*ones(1,length(wpnim(1,:)))).*wpnim(1,:);
    wpnm3 = sum(wpnm_temp3,1);    

    wpnp = wpnp1+ wpnp2+ wpnp3;  
    wpnm = wpnm1+ wpnm2+ wpnm3;  
    
    
    %rl and rl tilde computation
    %---------------------------
    wrnp = 2*wpnp - wxnp;
    wrnm = 2*wpnm - wxnm; 
    
    wrnp_cell = vect2cell(wrnp,rm,Ml,Mc,Nx);
    wrnm_cell = vect2cell(wrnm,rm,Ml,Mc,Nx);
    rn = idualtreeM2D_freq_deblur(wrnp_cell,wrnm_cell,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
    rnl = rn(:)';
    
    [Frlp_cell,Frlm_cell,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(rn,M,rm);
    Frlp = cell2vect(Frlp_cell,rm,Ml,Mc);
    Frlm = cell2vect(Frlm_cell,rm,Ml,Mc);    
    
    wrnp_ortho = wrnp - 1/mu*Frlp;  
    wrnm_ortho = wrnm - 1/mu*Frlm;   
    
        
    % yn update
    %----------    
    wynip_ortho = wynip_ortho + lambda*(ones(N1-1,1)*wrnp_ortho - wynip_ortho);
    wynim_ortho = wynim_ortho + lambda*(ones(N1-1,1)*wrnm_ortho - wynim_ortho);
    
    zni = zni + lambda*(ones(N1-1,1)*rnl - mu*qni);
        
    wynip(1,:) = wynip(1,:) + lambda*(ones(1,1)*wrnp - wpnip(1,:));
    wynim(1,:) = wynim(1,:) + lambda*(ones(1,1)*wrnm - wpnim(1,:));

   
    
    % xn update
    %----------
    wxnp = wxnp + lambda * (wpnp - wxnp);
    wxnm = wxnm + lambda * (wpnm - wxnm); 
        

    time(n+1) = time(n)+ toc;  
    
    
    %Transform domain to image domain
    %--------------------------------
    wprec = vect2cell(wxnp,rm,Ml,Mc,Nx);
    wmrec = vect2cell(wxnm,rm,Ml,Mc,Nx);
    xrec = idualtreeM2D_freq_deblur(wprec,wmrec,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
      
end
fprintf('\n\n PPXA Hybrid l1+tv:\n');
fprintf('----------\n');
fprintf('time = %5.1f  \n',time(n+1));
fprintf('snr = %3.2f\n',snr(n));
fprintf('ssim = %3.3f\n',ssim(x,xrec));
fprintf('dc = %3.2f  \n',dc(n));


figure
subplot(311)
plot(snr)
xlabel('SNR');
title('Hybrid restoration');
subplot(312)
plot(critere)
xlabel('Objective function');
subplot(313)
plot(dc)
xlabel('Distance to C');

%%%%%%%%%%%%%%%%% SAVE DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tit=['Hybrid_Poisson_TV_l1DTT-',nomImage,'-rm=',int2str(rm),'-niter',int2str(niter),'alpha',num2str(alpha),'L',int2str(L),'gamma',num2str(gamma),'lambda',num2str(lambda),'coef_TV',num2str(coef(2)),'coef_REG',num2str(coef(1)),'coef_App',num2str(coef_app),'.mat'];
save(tit,'x','z','xrec','snr','critere','time','divKL','sumcoef','tv','N1','dc','wprec','wmrec');
